"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const missing_config_key_error_1 = require("./missing-config-key-error");
const np = require('nested-property');
class ConfigFile {
    constructor(_filepath, template) {
        this._filepath = _filepath;
        this.template = template;
        this._data = undefined;
        this._isLoaded = false;
    }
    toJson() {
        return this._data;
    }
    all() {
        this.loadIfRequired();
        return this._data;
    }
    get(configName, defaultValue) {
        this.loadIfRequired();
        if (configName === undefined) {
            return this._data;
        }
        let value = np.get(this._data, configName);
        if (defaultValue === undefined && !value) {
            throw new missing_config_key_error_1.MissingConfigKeyError(configName);
        }
        return value || defaultValue;
    }
    delete(name, save = false) {
        let value = this.get(name);
        // np.delete(this._data, name);
        let parts = name.split(".");
        let lastKey = parts[parts.length - 1];
        let path = parts.slice(0, parts.length - 1).join('.');
        let ref;
        if (path) {
            ref = np.get(this._data, path);
        }
        else {
            ref = this._data;
        }
        delete ref[lastKey];
        return value;
    }
    set(configName, value, save = true) {
        this.loadIfRequired();
        np.set(this._data, configName, value);
        if (save) {
            this.save();
        }
        return this;
    }
    filepath() {
        return this._filepath;
    }
    exists() {
        return fs_1.existsSync(this._filepath);
    }
    touch() {
        this._data = this.template || {};
        this.save();
        return this;
    }
    save() {
        if (this._data === undefined) {
            throw new Error('Cannot save, file data is null');
        }
        // console.log(
        //     Colors.green("Saving configuration file: " + this._filepath)
        // );
        ConfigFile.writeFile(this._filepath, JSON.stringify(this._data, null, 4));
        return this;
    }
    loadIfRequired() {
        if (!this._isLoaded) {
            this.load();
        }
    }
    load() {
        if (!this.exists()) {
            this.touch();
        }
        this._data = JSON.parse(fs_1.readFileSync(this._filepath).toString());
        this._isLoaded = true;
        return this;
    }
    static writeFile(filename, content) {
        this.ensureDirectoryExistence(filename);
        fs_1.writeFileSync(filename, content);
    }
    static ensureDirectoryExistence(filePath) {
        var dirname = path_1.dirname(filePath);
        if (fs_1.existsSync(dirname)) {
            return;
        }
        this.ensureDirectoryExistence(dirname);
        fs_1.mkdirSync(dirname);
    }
}
exports.ConfigFile = ConfigFile;
//# sourceMappingURL=config-file.js.map